@@ -1,3 +1,4 @@
+#!/usr/bin/env python
 """
 Page output and find dimensions of console.
 
@@ -13,7 +14,7 @@ Author:  anatoly techtonik <techtonik@gmail.com>
 License: Public Domain (use MIT if Public Domain doesn't work for you)
 """
 
-__version__ = '1.0'
+__version__ = '1.1'
 
 import os,sys
 
@@ -153,12 +154,12 @@ def getch():
         # save old terminal settings, because we are changing them
         old_settings = termios.tcgetattr(fd)
         try:
-            # set terminal to "raw" mode, in which driver returns
+            # set terminal to "cbreak" mode, in which driver returns
             # one char at a time instead of one line at a time
             #
-            # tty.setraw() is just a helper for tcsetattr() call, see
+            # tty.setcbreak() is just a helper for tcsetattr() call, see
             # http://hg.python.org/cpython/file/c6880edaf6f3/Lib/tty.py
-            tty.setraw(fd)   # [ ] TODO - consider using setcbreak(fd)
+            tty.setcbreak(fd)
             ch = sys.stdin.read(1)
 
 
@@ -186,6 +187,16 @@ def getch():
 
     return ch
 
+def echo(msg):
+    """
+    Print msg to the screen without linefeed and flush the output.
+    
+    Standard print() function doesn't flush, see:
+    https://groups.google.com/forum/#!topic/python-ideas/8vLtBO4rzBU
+    """
+    sys.stdout.write(msg)
+    sys.stdout.flush()
+
 def prompt(pagenum):
     """
     Show default prompt to continue and process keypress.
@@ -193,9 +204,9 @@ def prompt(pagenum):
     It assumes terminal/console understands carriage return \r character.
     """
     prompt = "Page -%s-. Press any key to continue . . . " % pagenum
-    print prompt,
+    echo(prompt)
     getch()
-    print '\r' + ' '*(len(prompt)-1) + '\r',
+    echo('\r' + ' '*(len(prompt)-1) + '\r')
 
 def page(content, pagecallback=prompt):
     """
@@ -209,7 +220,11 @@ def page(content, pagecallback=prompt):
     pagenum = 1
 
     try:
-        line = content.next().rstrip("\r\n")
+        try:
+            line = content.next().rstrip("\r\n")
+        except AttributeError:
+            # Python 3 compatibility
+            line = content.__next__().rstrip("\r\n")
     except StopIteration:
         pagecallback(pagenum)
         return
@@ -217,16 +232,16 @@ def page(content, pagecallback=prompt):
     while True:     # page cycle
         linesleft = height-1 # leave the last line for the prompt callback
         while linesleft:
-            linelist = [line[i:i+width] for i in xrange(0, len(line), width)]
+            linelist = [line[i:i+width] for i in range(0, len(line), width)]
             if not linelist:
                 linelist = ['']
             lines2print = min(len(linelist), linesleft)
             for i in range(lines2print):
                 if os.name == 'nt' and len(line) == width:
                     # avoid extra blank line by skipping linefeed print
-                    print linelist[i],
+                    echo(linelist[i])
                 else:
-                    print linelist[i]
+                    print(linelist[i])
             linesleft -= lines2print
             linelist = linelist[lines2print:]
 
@@ -235,7 +250,11 @@ def page(content, pagecallback=prompt):
                 continue
             else:
                 try:
-                    line = content.next().rstrip("\r\n")
+                    try:
+                        line = content.next().rstrip("\r\n")
+                    except AttributeError:
+                        # Python 3 compatibility
+                        line = content.__next__().rstrip("\r\n")
                 except StopIteration:
                     pagecallback(pagenum)
                     return
@@ -243,110 +262,127 @@ def page(content, pagecallback=prompt):
         pagenum += 1
 
 
-if __name__ == '__main__':
-    print("Manual tests for pager module.")
-    # [ ] TODO - tests for getch() output
 
+# --- Manual tests when pager executed as a module ---
+
+def manual_test_console():
     # [ ] find appropriate term of 'console' for Linux
     print("\nconsole size: width %s, height %s" % (getwidth(), getheight()))
-    sys.stdout.write("--<enter>--")
+    echo("--<enter>--")
     getch()
-    print
+    echo("\n")
 
     print("\nsys.stdout.write() doesn't insert newlines automatically,")
     print("that's why it is used for console output in non-trivial")
-    print("cases here.")
-    print
+    print("cases here.\n")
     sys.stdout.write("--<enter>--")
+    sys.stdout.flush()
+    getch()
+    print("\rHowever, sys.stdout.write() requires explicit flushing")
+    print("to make the output immediately appear on the screen.")
+    print("echo() function from this module does this automatically.")
+    echo("\n--<enter>--")
     getch()
-    print
 
-    print("\nThe following test outputs string equal to the width of the\n"
+    print("\n\nThe following test outputs string equal to the width of the\n"
           "screen and waits for you to press <enter>. It behaves\n"
           "differently on Linux and Windows - W. scrolls the window and\n"
           "places cursor on the next line immediately, while L. window\n"
-          "doesn't scroll until the next character is output.")
-    print
+          "doesn't scroll until the next character is output.\n"
+         )
     print("Tested on:")
     print("  Windows Vista - cmd.exe console")
     print("  Debian Lenny - native terminal")
     print("  Debian Lenny - PuTTY SSH terminal from Windows Vista")
-    print
-    sys.stdout.write("--<enter>--")
+    echo("\n--<enter>--")
     getch()
-    print
+    echo("\n")
 
-    sys.stdout.write("<" + "-"*(getwidth()-2) + ">")
+    echo("<" + "-"*(getwidth()-2) + ">")
     getch()
     print("^ note there is no newline when the next character is printed")
-    print
+    print("")
     print("At least this part works similar on all platforms. It is just\n"
           "the state of the console after the last character on the line\n"
           "is printed that is different.")
-    print
-    sys.stdout.write("--<enter>--")
+    print("")
+    echo("--<enter>--")
     getch()
-    print
+    print("")
 
     print("\nBut there is one special case.")
-    print
+    print("")
     print("It is when the next character is a newline.")
-    print
+    print("")
     print("The following test prints line equal to the width of the\n"
           "console, waits for <enter>, then outputs newline '\\n',\n"
           "waits for another key press, then outputs 'x' char.")
-    print
-    sys.stdout.write("--<enter>--")
+    print("")
+    echo("--<enter>--")
     getch()
-    print
+    print("")
 
-    sys.stdout.write("<" + "-"*(getwidth()-2) + ">")
+    echo("<" + "-"*(getwidth()-2) + ">")
     getch()
-    sys.stdout.write("\n")
+    echo("\n")
     getch()
-    sys.stdout.write("x")
+    echo("x")
     getch()
 
     print("\n^ here is the difference:")
-    print
+    print("")
     print("On Windows you will get:\n"
           "  <----------->\n"
           "  \n"
           "  x")
-    print
+    print("")
     print("Linux will show you:\n"
           "  <----------->\n"
           "  x")
-    print
-    sys.stdout.write("--<enter>--")
+    print("")
+    echo("--<enter>--")
     getch()
-    print
+    print("")
 
     print("\nThe next test will fill the screen with '1' digits\n"
           "numbering each line staring from 1.")
-    print
+    print("")
     print("It works the same on Linux and Windows, because the next\n"
           "character after the last on the line is not linefeed.\n")
-    sys.stdout.write("--<enter>--")
+    echo("--<enter>--")
     getch()
-    print
+    print("")
     numwidth = len(str(getwidth()))
     strlen = getwidth() - numwidth - 2 # 2 = '. ' after the line number
     filler = '1' * strlen
     for i in range(getheight()-1):     # -1 to leave last line for --<enter>--
         lineno = ("%" + str(numwidth) + "s. ") % (i+1)
         sys.stdout.write(lineno + filler)
-    sys.stdout.write("--<enter>--")
+    echo("--<enter>--")
     getch()
-    print
+    print("")
 
     print("\nNext test prints this source code using page() function")
-    print
-    sys.stdout.write("--<enter>--")
+    print("")
+    echo("--<enter>--")
     getch()
-    print
+    print("")
     content = open(__file__)
     page(content)
-    sys.stdout.write("--<enter>--")
+    echo("--<enter>--")
     getch()
-    print
+    print("")
+
+
+if __name__ == '__main__':
+    print("Manual tests for pager module.")
+    # [ ] TODO - tests for getch() output
+    ch = ''
+    while ch != '0':
+      print("\n1. Test output")
+      print("2. Test input")
+      print("0. Exit")
+      ch = getch()
+      if ch == '1':
+        manual_test_console()
+
